@@ -1533,7 +1533,20 @@ domain & limit[1] = %s
     def __getitem__(self, indices):
         return self.func(self.expr[indices], *self.limits)
 
+    def _eval_torch(self):
+        expr = self.expr
+        limits = self.limits
+        
+        ranges = []
+        for x, *args in limits:
+            assert x.is_integer
+            ranges.append([t.torch for t in args])
 
+        from sympy.concrete.expr_with_limits import Lamda
+        self = Lamda(expr, *limits).torch
+        return sum(self[indices] for indices in itertools.product(*(range(*args) for args in ranges)))
+    
+    
 def summation(f, *symbols, **kwargs):
     r"""
     Compute the summation of f with respect to symbols.
