@@ -1159,7 +1159,7 @@ class Add(Expr, AssocOp):
                     rhs = -rhs
                     return self.func(lhs, rhs).simplify()
 
-    def simplifyKroneckerDelta(self): 
+    def simplify_KroneckerDelta(self): 
         dic = {}
        
         for expr in self.enumerate_KroneckerDelta():
@@ -1205,11 +1205,11 @@ class Add(Expr, AssocOp):
             else:
                 coefficient = _coefficient
                 if coefficient.is_Add:
-                    coefficient = coefficient.simplifyKroneckerDelta()     
+                    coefficient = coefficient.simplify_KroneckerDelta()     
                         
             this = coefficient * delta + p.nth(0)
             if this.is_Add:
-                this = this.simplifyKroneckerDelta()
+                this = this.simplify_KroneckerDelta()
             
         return this
 
@@ -1233,7 +1233,7 @@ class Add(Expr, AssocOp):
                 return this.simplify(deep=deep)
             return this         
 
-        this = self.simplifyKroneckerDelta()
+        this = self.simplify_KroneckerDelta()
         if this != self:
             return this
 
@@ -1773,10 +1773,54 @@ class Add(Expr, AssocOp):
             
         return S.One, self
         
-    @cacheit
-    def torch(self):
-        return sum(arg.torch() for arg in self.args)
+    def _eval_torch(self):
+        return sum(arg.torch for arg in self.args)
 
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        vars = self.variables
+        limits = self.limits
+        modified = []
+        unmodified = []
+        constants = []
+        for arg in self.expr.args:
+            if arg.has(*vars):
+                if arg.is_Mul:
+                    unmodified.append(arg)
+                else:
+                    lamda = self.func(arg, *limits)
+                    _lamda = lamda.simplify()
+                    if _lamda != lamda:
+                        modified.append(_lamda)
+                    else:
+                        unmodified.append(arg)
+            else:
+                constants.append(arg)
+    
+        if modified or constants:
+            if unmodified:
+                if max(len(expr.shape) for expr in unmodified) < len(self.expr.shape):
+                    return self
+                
+                if len(unmodified) == 1:
+                    unmodified, = unmodified
+                    if unmodified.is_Mul:
+                        lamda = self.func(unmodified, *limits)
+                        _lamda = lamda.simplify()
+                        if _lamda != lamda:
+                            modified.append(_lamda)
+                            unmodified = S.Zero
+                            
+                    if unmodified:
+                        unmodified = self.func(unmodified, *limits)
+                else:
+                    unmodified = self.func(Add(*unmodified), *limits)
+            else:
+                unmodified = S.Zero
 
+            return Add(*modified, *constants, unmodified)
+        return self
+    
+    
 from .mul import Mul, _keep_coeff, prod
 from sympy.core.numbers import Rational
