@@ -1237,6 +1237,8 @@ class Symbol(AtomicExpr, NotIterable):
     def __hash__(self):
         return super(Symbol, self).__hash__()
 
+    nonboolean_attributes = {'domain', 'shape', 'etype', 'distribution'}
+    
     def __eq__(self, other):
         try:
             other = sympify(other)
@@ -1256,8 +1258,7 @@ class Symbol(AtomicExpr, NotIterable):
         if definition is not None:
             return True
                 
-        nonboolean_attributes = {'domain', 'shape', 'etype', 'distribution'}
-        for attr in nonboolean_attributes:
+        for attr in Symbol.nonboolean_attributes:
             if (attr in self._assumptions) != (attr in other._assumptions):
                 return False
         
@@ -1265,11 +1266,11 @@ class Symbol(AtomicExpr, NotIterable):
             if self._assumptions[fact] != other._assumptions[fact]:
                 return False
 
-        for fact in self._assumptions.keys() - other._assumptions.keys() - nonboolean_attributes: 
+        for fact in self._assumptions.keys() - other._assumptions.keys() - Symbol.nonboolean_attributes: 
             if other._ask(fact) != self._assumptions[fact]:
                 return False
             
-        for fact in other._assumptions.keys() - self._assumptions.keys() - nonboolean_attributes:
+        for fact in other._assumptions.keys() - self._assumptions.keys() - Symbol.nonboolean_attributes:
             if self._ask(fact) != other._assumptions[fact]:
                 return False
 
@@ -1385,24 +1386,41 @@ class Symbol(AtomicExpr, NotIterable):
         """Overloading for <<"""
         return Assuming(self, other)
     
-    @cacheit
+    @property
     def torch(self):
         definition = self.definition
         if definition is None:
-            import torch
-            data = torch.from_numpy(self.data)
-            if torch.cuda.is_available():
-                data = data.cuda()
-            return data
-        return definition.torch()
+            try:
+                return self._torch
+            except AttributeError:
+                import torch
+                data = torch.from_numpy(self.numpy)
+                if torch.cuda.is_available():
+                    data = data.cuda()
+
+                self._torch = data
+                return data
+        
+        return definition.torch
 
     def find_path(self, cls, path, **kwargs):
-        if kwargs.get('definition'):
+        for attr in kwargs:
+            value = getattr(self, attr)
+            if value is not None:
+                path.append(attr)
+                yield from value.find_path(cls, path, **kwargs)
+                path.pop()
+            
+    def _assumptions_clear_cache(self, cacheType, sym):
+        if self._has(sym):
+            try:
+                self.__delattr__(cacheType)
+            except AttributeError:
+                ...
+            
             definition = self.definition
             if definition is not None:
-                path.append('definition')
-                yield from definition.find_path(cls, path, **kwargs)
-                path.pop()
+                definition._assumptions_clear_cache(cacheType, sym)
             
     
 class Dummy(Symbol):
