@@ -928,7 +928,7 @@ class Piecewise(Function):
         # somewhat more efficient by avoiding other substitutions
         # and avoiding invalid conditions that appear after a
         # True condition
-        args = []        
+        args = []
         hit = False
         from sympy.core.basic import _aresame
         
@@ -1719,11 +1719,60 @@ class Piecewise(Function):
                 if expr.is_Piecewise:
                     expr.check_sanctity()
          
-    @cacheit
-    def torch(self):
+    def _eval_torch(self):
         for expr, cond in self.args:
-            if cond.torch() == True:
-                return expr.torch()
+            if cond.torch == True:
+                return expr.torch
+    
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        limits_dict = self.limits_dict
+        if len(limits_dict) > 1:
+            expr = self.func(self.expr, self.limits[0]).simplify(squeeze=squeeze)
+            if not expr.is_Lamda:
+                return self.func(expr, *self.limits[1:]).simplify(squeeze=squeeze)
+            if not expr.expr.is_Piecewise:
+                return self.func(expr.expr, *expr.limits, *self.limits[1:])
+                
+        else:
+            if len(self.expr.args) == 2:
+                e0, c0 = self.expr.args[0]
+                if c0.is_Element:
+                    e, s = c0.args
+                    if e in limits_dict.keys():
+                        if s.is_Complement:
+                            U, A = s.args
+                            domain, *_ = limits_dict.values()
+                            if domain in U:
+                                e1, _ = self.expr.args[1]
+                                from sympy.sets.contains import Element
+                                expr = self.expr.func((e1, Element(e, A)), (e0, True)).simplify()
+                                return self.func(expr, *self.limits).simplify(squeeze=squeeze)
+                        elif s.is_Range:
+                            if limits_dict[e] in s:
+                                return self.func(e0, *self.limits).simplify(squeeze=squeeze)
+            if self.expr.is_set:
+                return self
+            
+            args = []
+            for e, c in self.expr.args:
+                if not e._has(self.variable):
+                    return self
+                
+                if c._has(self.variable):
+                    return self
+                
+                this_old = self.func(e, *self.limits)
+                this = this_old.simplify()
+                if this == this_old:
+                    return self
+                
+                args.append((this, c))
+                
+            return self.expr.func(*args)
+        
+        return self
+                
     
 def piecewise_fold(expr):
     """
