@@ -1,8 +1,9 @@
 import math
 import warnings
 import numbers
-from typing import List, Tuple, Optional, overload, Union, cast
+from typing import Tuple, Union, cast
 
+from ... import nn
 from .module import Module
 
 from sympy.core.symbol import Symbol
@@ -17,7 +18,7 @@ from sympy.concrete.expr_with_limits import Lamda
 from sympy.matrices.expressions.transpose import Transpose
 from sympy.keras.math import logsumexp
 from sympy.matrices.expressions.blockmatrix import BlockMatrix
-from sympy.keras import nn
+
 
 def apply_permutation(tensor, permutation, dim: int = 1):
     return tensor.index_select(dim, permutation)
@@ -1200,7 +1201,7 @@ class CRF(Module):
 
         i = Symbol(integer=True)
         
-        @Function(real=True, shape=(batch_size, units))
+        @Function(real=True, shape=(batch_size, units), elementwise=False)
         def forward(G, x, *limits):
             [[i]] = limits
             _score = forward[i - 1](G, x)
@@ -1233,7 +1234,7 @@ class CRF(Module):
 
         out_of_bound = ZeroMatrix(batch_size)#the true labeled is padded with zeros, not -1!
         
-        @Function(real=True, shape=(batch_size,))
+        @Function(real=True, shape=(batch_size,), elementwise=False)
         def backward(argmax_tables, *limits):
             [[i]] = limits
             next_argmax = backward[i - 1](argmax_tables)
@@ -1274,7 +1275,7 @@ class CRF(Module):
         G = chain_kernel.unsqueeze(0, batch_size)
 
         units = self.units
-        @Function(real=True, shape=(batch_size, units))
+        @Function(real=True, shape=(batch_size, units), elementwise=False)
         def forward(x, *limits):
             [[i]] = limits
             minus_x = forward[i - 1](G, x)
