@@ -244,6 +244,19 @@ class MatrixExpr(Expr):
                 i, j = _sympify(i), _sympify(j)
                 assert self.valid_index(i, j) != False
                 return self._entry(i, j, expand=False)
+            else:
+                i, j, *indices = key
+                self = self._entry(i, j)
+                if isinstance(i, slice):
+                    if isinstance(j, slice):
+                        return self._entry_multiple(slice(None), slice(None), *indices)
+                    else:
+                        return self[tuple((slice(None), *indices))]
+                else: 
+                    if isinstance(j, slice):
+                        return self[tuple((slice(None), *indices))]
+                    else:
+                        return self[tuple(indices)]
                 
         assert isinstance(key, (int, slice)) or key.is_Expr
         return self._entry(key)
@@ -1035,6 +1048,24 @@ class ZeroMatrix(ConstantMatrix, MatrixExpr):
     def doit(self, **hints):
         return self
     
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        if squeeze:
+            return S.Zero
+        
+        shape = []
+        for v, *ab in self.limits:
+            if len(ab) == 2:
+                a, b = ab
+                size = b - a
+            elif not ab:
+                from sympy.sets.fancysets import Range
+                a, b = v.domain.of(Range)
+                size = b - a
+            shape.append(size)
+        shape.reverse()
+        return ZeroMatrix(*shape, *self.expr.shape)
+    
 
 class GenericZeroMatrix(ZeroMatrix):
     """
@@ -1155,6 +1186,24 @@ class OneMatrix(ConstantMatrix, MatrixExpr):
     def _eval_is_extended_integer(self):
         return True
 
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        if squeeze:
+            return S.One
+        
+        shape = []
+        for v, *ab in self.limits:
+            if len(ab) == 2:
+                a, b = ab
+                size = b - a
+            elif not ab:
+                from sympy.sets.fancysets import Range
+                a, b = v.domain.of(Range)
+                size = b - a
+            shape.append(size)
+        shape.reverse()
+        return OneMatrix(*shape, *self.expr.shape)
+    
     
 def matrix_symbols(expr):
     return [sym for sym in expr.free_symbols if sym.is_Matrix]
