@@ -172,6 +172,37 @@ class MatMul(MatrixExpr):
             shape = self.matmul_shape(shape, self.args[i].shape)
         return shape
 
+    def _entry_multiple(self, i, j=None, *rest):
+        len_shape = self.max_len_shape()
+        args = [*self.args]
+        for t, arg in enumerate(args):
+            if len(arg.shape) == len_shape:
+                args[t] = arg[i]
+        
+        if isinstance(i, slice):
+            if len_shape > 3:
+                for t, arg in enumerate(args):
+                    if len(arg.shape) == len_shape:
+                        args[t] = arg[:, j]
+                    elif len(arg.shape) == len_shape - 1:
+                        args[t] = arg[j]
+
+                raise Exception("unresolved case")
+            else:
+                if len(args[0].shape) == len_shape:
+                    args[0] = args[0][:, j]
+                else:
+                    for t, arg in enumerate(args):
+                        if len(arg.shape) == len_shape:
+                            args[t] = arg[:, j]
+                            break
+                        
+                args[-1] = args[-1][tuple(*[slice(None)] * (len(args[-1].shape) - 1), *rest)]
+            
+            return MatMul(*args)
+        else:
+            return MatMul(*args)[tuple(j, *rest)]
+        
     def _entry(self, i, j=None, expand=True, **kwargs):
         if j is None:
             if len(self.args[0].shape) == 1:
@@ -180,10 +211,38 @@ class MatMul(MatrixExpr):
             if len(rhs.shape) == 3:
                 rhs = rhs[i]
             return self.args[0][i] @ rhs
+        
+        if len(self.shape) > 2:
+            len_shape = self.max_len_shape()
+            args = [*self.args]
+            for t, arg in enumerate(args):
+                if len(arg.shape) == len_shape:
+                    args[t] = arg[i]
+            
+            if isinstance(i, slice):
+                if len_shape > 3:
+                    for t, arg in enumerate(args):
+                        if len(arg.shape) == len_shape:
+                            args[t] = arg[:, j]
+                        elif len(arg.shape) == len_shape - 1:
+                            args[t] = arg[j]
+                else:
+                    if len(args[0].shape) == len_shape:
+                        args[0] = args[0][:, j]
+                    else:
+                        for t, arg in enumerate(args):
+                            if len(arg.shape) == len_shape:
+                                args[t] = arg[:, j]
+                                break
+                            
+                return MatMul(*args)
+            else:
+                return MatMul(*args)[j]
+        
         if isinstance(i, slice):
-            start, stop = i.start, i.stop
+            start, stop, step = i.start, i.stop, i.step
             if start is None:
-                if stop is None:
+                if stop is None and step is None:
                     return self.func(*self.args[:-1]) @ self.args[-1][:, j]
                 start = 0
             if stop is None:
@@ -623,13 +682,51 @@ class MatMul(MatrixExpr):
     def _subs(self, old, new, **hints):
         if old.is_MatMul:
             args = old.args
-            from sympy.utilities.misc import sunday
+            from std.search import sunday
             i = sunday(self.args, args)
             if i >= 0:
                 return self.func(*self.args[:i] + (new.args if new.is_MatMul else (new,)) + self.args[i + len(args):]).simplify()
         return MatrixExpr._subs(self, old, new, **hints)
 
+    def _eval_torch(self):
+        from functools import reduce
+        return reduce(lambda x, y: x @ y if len(y.shape) > 1 else (x @ y.unsqueeze(-1)).squeeze(-1), (arg.torch for arg in self.args))
+    
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        expr = self.expr
+        var = self.limits[0]
+        *former, last = expr.args 
+        if last._has(var) and not any(e._has(var) for e in former):
+            last = self.func(last, var)
+            _last = last.simplify()
+            if last != _last:
+                if len(_last.shape) == 2:
+                    _last = _last.T
+                independent = expr.func(*former, _last).simplify()
+                limits = self.limits[1:]
+                if limits:
+                    independent = self.func(independent, *limits)
+                return independent
+            
+        var = self.limits[-1][0]
+        
+        first, *latter = expr.args
+        if first._has(var) and not any(arg._has(var) for arg in latter):
+# consider the complex case:
+# [i:n - Min(l, n) + 1](Q[i + Min(l, n) - 1] @ W @ K[i:i + Min(l, n)].T)
+            first = self.func(expr.args[0], self.limits[-1])
+            _first = first.simplify()
+            if first != _first:
+                independent = expr.func(_first, *latter).simplify()
+                limits = self.limits[:-1]
+                if limits:
+                    independent = self.func(independent, *limits)
+                return independent
 
+        return self
+        
+    
 def validate(*matrices):
     """ Checks for valid shapes for args of MatMul """
     for i in range(len(matrices) - 1):
