@@ -488,7 +488,7 @@ class Indexed(Expr):
             return self.base._has(pattern)
         else: 
             if self.base.is_AppliedUndef:
-                return any(arg._has(pattern) for arg in self.base.args)
+                return self.base._has(pattern)
                                 
             definition = self.definition
             if definition is not None:
@@ -760,6 +760,157 @@ class Indexed(Expr):
         
         return self
                                 
+    def _eval_torch(self):
+        base, *indices = (arg.torch for arg in self.args)
+        return base[indices]
+    
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        variables = tuple(x for x, *_ in self.limits[::-1])
+        expr = self.expr
+        if expr.is_set:
+            if expr.args[-len(variables):] == variables:
+                base = expr.base[expr.indices[:-len(variables)]]
+                for x, *ab in reversed(self.limits):
+                    if len(ab) == 1:
+                        return self
+                    
+                    if len(ab) == 2:
+                        a, b = ab
+                        base = base[a:b]
+                        
+                    else:
+                        domain = x.domain
+                        if domain.is_Range and domain.start == 0 and domain.stop == base.shape[0]:
+                            continue
+                        
+                        return self
+                        
+                return base
+        else:
+            indices = expr.indices
+            if len(variables) == 1:
+                var, = variables
+                try:
+                    i = indices.index(var)
+                    _, *ab = self.limits[0]
+                    if ab :
+                        if len(ab) == 2:
+                            if i == len(indices) - 1:
+                                indexed = expr.base[indices[:-1]]
+                                zero, b = ab
+                                assert zero.is_zero
+                                shape = b
+                                if expr.base.shape[0] != shape:
+                                    return indexed[:shape]
+                                return indexed
+                            else:
+                                indexed = expr.base[indices[:i]]
+                                zero, b = ab
+                                assert zero.is_zero
+                                shape = b
+                                
+                                rest_indices = indices[i + 1:]
+                                return indexed[(slice(0, shape),) + rest_indices]
+                                                            
+                        else:
+                            [domain] = ab
+                            if i == len(indices) - 1:
+                                indexed = expr.base[indices[:-1]]
+                                start, stop, step = domain.args
+                                if start.is_zero:
+                                    if step.is_One:
+                                        if expr.base.shape[0] != stop:
+                                            return indexed[:stop]
+                                        return indexed
+                                    else:
+                                        return indexed[:stop:step]
+                                else:
+                                    if step.is_One:
+                                        return indexed[start:stop]
+                                    else:
+                                        return indexed[start:stop:step]
+                            else:
+                                indexed = expr.base[indices[:i]]
+                                rest_indices = indices[i + 1:]
+                                start, stop, step = domain.args
+                                if start.is_zero:
+                                    if step.is_One:
+                                        return indexed[(slice(0, stop),) + rest_indices]
+                                    else:
+                                        return indexed[(slice(0, stop, step),) + rest_indices]
+                                else:
+                                    if step.is_One:
+                                        return indexed[(slice(start, stop),) + rest_indices]
+                                    else:
+                                        return indexed[(slice(start, stop, step),) + rest_indices]
+                                
+                    
+                except ValueError:
+                    validIndices = [(i, index) for i, index in enumerate(expr.indices) if index._has(var)]
+                    if len(validIndices) == 1:
+                        [[i, index]] = validIndices
+                        # index = step * var + start
+                        start, step = index.of_simple_poly(var)
+                        if step == 1:
+                            _, *ab = self.limits[0]
+                            if ab:
+                                if len(ab) == 2:
+                                    zero, shape = ab
+                                    assert zero.is_zero
+                                    step = 1
+                                else:
+                                    [domain] = ab
+                                    zero, shape, step = domain.args
+                                    
+                                return expr.base[indices[:i] + (slice(start, shape + start, step),) + indices[i + 1:]]
+                            
+                        elif step:
+                            _, *ab = self.limits[0]
+                            if ab:
+                                if len(ab) == 2:
+                                    zero, shape = ab
+                                    assert zero.is_zero
+                                    if shape.is_Ceiling:
+                                        stop = shape.arg * step + start
+                                        if not stop._has(var):
+                                            return expr.base[indices[:i] + (slice(start, stop, step),) + indices[i + 1:]]
+                
+            if expr.args[-len(variables):] == variables:
+                return expr.base[indices[:-len(variables)]]
+    
+            [*indices] = indices
+            j = len(indices)
+            
+            hit = False
+            for i, var in enumerate(self.variables):
+                for j in range(j - 1, -1, -1):
+                    if indices[j] == var:
+                        break
+                else:
+                    break
+                
+                hit = True
+                ab = self.limits[i][1:]
+                if len(ab) == 2:
+                    a, b = ab
+                    if a == 0:
+                        indices[j] = slice(b)
+                    else:
+                        indices[j] = slice(*ab)
+                elif not ab:
+                    indices[j] = slice(self.shape[-i - 1])
+                else:
+                    [domain] = ab
+                    indices[j] = slice(*domain.args)
+            else:
+                i += 1
+                
+            if hit:
+                return expr.base[indices]
+                
+        return self
+        
         
 class Sliced(Expr):
     """Represents a mathematical object with Slices,
@@ -1320,6 +1471,9 @@ class Sliced(Expr):
         if self.base.is_AppliedUndef:
             return any(arg._has(pattern) for arg in self.base.args)
 
+        if self.base.is_Indexed and self.base._has(pattern):
+            return True
+        
         definition = self.base.defun()
         if definition is not None: 
             if any(index._has(pattern) for index in self.indices):
@@ -1540,6 +1694,31 @@ class Sliced(Expr):
             
         return self
 
+    def _eval_torch(self):
+        base, *indices = self.args
+        base = base.torch
+        indices = [slice(*[None if arg is None else arg.torch for arg in index.args]) for index in indices]
+        return base[indices]
+    
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        (var, *ab), *limits = self.limits
+        base, *slices = self.expr.args
+        if base.is_Indexed:
+            base, *indices = base.args
+            if indices[-1] == var:
+                if len(indices) > 1:
+                    ...
+                else:
+                    if len(ab) == 2:
+                        a, b = ab
+                        if b.is_integer:
+                            if not limits:
+                                if not any(s._has(var) for s in slices):
+                                    return Sliced(base, Tuple(a, b), *slices)
+            
+        return self
+    
 
 class SlicedIndexed(Expr):
     """Represents a mathematical object with Slices and indices both.
@@ -2104,6 +2283,40 @@ class SlicedIndexed(Expr):
                 return new[self_start - start: self_stop - start]
         return Expr._subs(self, old, new, **kwargs)
 
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        expr = self.expr
+        slices, indices = expr.slices_indices()
+        if len(indices) == 1:
+            [var] = indices
+            _var, *ab = self.limits[0]
+            if var == _var:
+                base = expr.base
+                if not ab or ab == [0, base.shape[-1]]:
+                    if slices[-1].is_Tuple:
+                        start, stop, *_ = slices[-1]
+                        if start == 0 and stop == base.shape[-2]:
+                            expr = base[slices[:-1]].T
+                            if limits := self.limits[1:]:
+                                expr = self.func(expr, *limits)
+                            return expr
+                        
+                    else:
+                        start, stop, step = slices[-1].args
+                        if step.is_One and start == 0 and stop == base.shape[-2]:
+                            expr = base[slices[:-1]].T
+                            if limits := self.limits[1:]:
+                                expr = self.func(expr, *limits)
+                            return expr
+                        
+        return self
+
+    def _eval_torch(self):
+        base = self.base.torch
+        slices = [slice(*[None if arg is None else arg.torch for arg in index.args]) for index in self.slices]
+        indices = [index.torch for index in self.indices]
+        return base[tuple((*slices, *indices))]
+    
 
 # Warning: the following class is obsolete!!! using Symbol.x(shape=(m,n,k)) instead
 class IndexedBase(Expr, NotIterable):
